from distutils.core import setup
setup(
  name = 'zennla',
  packages = ['zennla'],
  version = '0.4',
  description = 'A REST microframework for Google App Engine',
  author = 'Abhinav Desor',
  author_email = 'abhinav.desor@gmail.com',
  url = 'https://github.com/adesor/zenn-la',
  download_url = 'https://github.com/adesor/zenn-la/tarball/0.4',
  keywords = ['google app engine', 'app engine', 'gae', 'ndb', 'webapp2', 'rest', 'framework'],
  classifiers = [],
)
